/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * Desk clock "Электроника Альфа 12-41А"
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "digits.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
volatile flag_t Flag = {0};
static rtc_t Clock;

/* Private function prototypes -----------------------------------------------*/
void bip_Bip(void);
static void dots_Off(void);
static void Show_MMSS(void);

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
  i2c_status_t i2c_status;

  /* Initialize onBoard Hardware */
  Board_Init();

  /* Initialize Scheduler */
  RTOS_Init();

  display_Init();
  display_String("Electronica Alpha-12\0", 1);

  /* Real-Time clock */
  i2c_status = RTC_Init();
  if (i2c_status != I2C_Ret_OK) {
    bip_Bip();
    display_String("RTC_Init Error\0", 1);
  }
  i2c_status = RTC_ReadTime(&Clock);
  if (i2c_status != I2C_Ret_OK) {
    bip_Bip();
    display_String("RTC_ReadTime Error\0", 1);
  }

  Show_MMSS();

  /* Infinite loop */
  while (1)
  {
    if (Flag.RTC_IRQ != 0) {
      Flag.RTC_IRQ = 0;
      i2c_status = RTC_ReadTime(&Clock);
      if (i2c_status == I2C_Ret_OK) {
        Show_MMSS();
      } else {
        display_String("I2C Err\0", 1);
      }
    }

    RTOS_DispatchTask();

    __WFI();
  }
}

/* Private functions ---------------------------------------------------------*/

/**
 * @brief 
 * Звук "пи-пи...пи-пи...":
 *  - две посылки по 125 мс,
 *  - пауза внутри посылки 63 мс,
 *  - пауза между посылками 689 мс,
 *  - частота заполнения 4 кГц,
 *  - период 1 секунда.
 * 
 */
void bip_Bip(void) {
  static int stat = 0;

  switch (stat) {
  case 0:
    stat = 1;
    BUZER_ON;
    RTOS_SetTask(bip_Bip, 100, 0);
    break;

  case 1:
    stat = 2;
    BUZER_OFF;
    RTOS_SetTask(bip_Bip, 75, 0);
    break;

  case 2:
    stat = 3;
    BUZER_ON;
    RTOS_SetTask(bip_Bip, 150, 0);
    break;

  case 3:
    stat = 0;
    BUZER_OFF;
    //RTOS_SetTask(bip_Bip, 675, 0);
    break;

  default:
    BUZER_OFF;
    stat = 0;
    break;
  }
}

/**
 * @brief Show MM:SS use 4x7 digits
 */
static void Show_MMSS(void) {
  uint8_t t;
  // dots on
  display_WriteData(&digit_4x7[10][0], 10, 4);
  RTOS_SetTask(dots_Off, 500, 0);

  t = Clock.Min >> 4;
  display_WriteData(&digit_4x7[t][0], 1, 4);

  t = Clock.Min & 0xf;
  display_WriteData(&digit_4x7[t][0], 6, 4);

  t = Clock.Sec >> 4;
  display_WriteData(&digit_4x7[t][0], 14, 4);

  t = Clock.Sec & 0xf;
  display_WriteData(&digit_4x7[t][0], 19, 4);

  display_WriteBuffer();
}

static void dots_Off(void) {
  display_Buffer[11] = 0x0;
  display_Buffer[12] = 0x0;
  display_WriteBuffer();
}

/******************************************************************************/
/*            Cortex-M0 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}
